from typing import Type
import pygame
import json
import random
from python.utils import *
from python.items import Item
from pyautogui import confirm, alert


INV_EMPTY = (23, 88, 101)
INV_NOT_EMPTY = (46, 209, 239)
INV_SELECTED = (255,215,0)
INV_HOVER = (0, 215, 255)
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)

class NPCS():
    def __init__(self):
        self.load_npcs()
        self.npcs_spawned = []

    def update(self, dt, player, scroll):
        interactions = []
        for npc in self.npcs_spawned:
            x = npc.update(dt, player, scroll)
            interactions.append(x)
        for x in interactions:
            if x == True:
                return True
            elif x == False:
                return False
        return None

    def render(self, win, scroll, dt, time, show_outline):
        for npc in self.npcs_spawned:
            npc.render(win, scroll, dt, time, show_outline)

    def handle_keys(self, key):
        interactions = []
        for npc in self.npcs_spawned:
            if npc.on_key != None:
                x = npc.on_key(key)
                interactions.append(x)
        for x in interactions:
            if x == True:
                return True
            elif x == False:
                return False
        return None
        

    def handle_click(self, btn, inventory, player):
        for npc in self.npcs_spawned:
            if npc.on_click != None:
                npc.on_click(btn, inventory, player)
    
    def load_npcs(self):
        with open("assets/json/npcs.json", "r") as f:
            self.npcs = json.load(f)
        self.art = {}
        for npc in self.npcs:
            art_folder = self.npcs[npc]["art folder"]
            idle_anim_name = self.npcs[npc]["idle anim"]
            walk_anim_name = self.npcs[npc]["walk anim"]
            self.art[npc] = {}
            self.art[npc]["animations"] = [[], [], [], []]
            for x in range(self.npcs[npc]["frames"]):
                idle_right = pygame.transform.scale(pygame.image.load(f"{art_folder}/{idle_anim_name}{x+1}.png").convert_alpha(), self.npcs[npc]["dimensions"])
                walk_right = pygame.transform.scale(pygame.image.load(f"{art_folder}/{walk_anim_name}{x+1}.png").convert_alpha(), self.npcs[npc]["dimensions"])
                idle_left = pygame.transform.flip(idle_right, True, False)
                walk_left = pygame.transform.flip(walk_right, True, False)
                self.art[npc]["animations"][0].append(idle_right)
                self.art[npc]["animations"][1].append(idle_left)
                self.art[npc]["animations"][2].append(walk_right)
                self.art[npc]["animations"][3].append(walk_left)


class NPC():
    def __init__(self, x, y, id, npc, animations, items):
        self.items = items
        self.id = id
        self.npc = npc
        self.name = self.npc["name"]
        self.width = self.npc["dimensions"][0]
        self.height = self.npc["dimensions"][1]
        self.speed = self.npc["speed"]
        self.x = x
        self.y = y
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        animations = animations["animations"]
        self.anim_idle_right = animations[0]
        self.anim_idle_left = animations[1]
        self.anim_walk_right = animations[2]
        self.anim_walk_left = animations[3]
        self.frames = self.npc["frames"]
        self.frame = 0
        self.last_direction = "right"
        self.last_y_direction = None
        self.moving = False
        self.move_x = True
        if self.npc["extra functions"]:
            self.extra_functions = ExtraFunctions(self)
            self.check_special_functions()

    def check_special_functions(self):
        self.on_update = None
        self.on_render = None
        self.on_init = None
        self.on_click = None
        self.on_key = None
        this = self.extra_functions
        if "onUpdate" in self.npc:
            self.on_update = getattr(this, self.npc["onUpdate"])
        if "onRender" in self.npc:
            self.on_render = getattr(this, self.npc["onRender"])
        if "onMouseClick" in self.npc:
            self.on_click = getattr(this, self.npc["onMouseClick"])
        if "onKeyPress" in self.npc:
            self.on_key = getattr(this, self.npc["onKeyPress"])
        if "onInit" in self.npc:
            getattr(this, self.npc["onInit"])()

    def update(self, dt, player, scroll):
        if self.on_update != None:
            x = self.on_update(dt, player, scroll)
            return x
        return None

    def move(self):
        if self.moving:
            if self.move_x:
                if self.last_direction == "right":
                    self.x += self.speed
                    self.rect.x += self.speed
                elif self.last_direction == "left":
                    self.x -= self.speed
                    self.rect.x -= self.speed
            if self.last_y_direction != None:
                if self.last_y_direction == "up":
                    self.y -= self.speed
                    self.rect.y -= self.speed
                elif self.last_y_direction == "down":
                    self.y += self.speed
                    self.rect.y += self.speed

    def isInLine(self, entity, range): # Checks if entity is in given range
        entityDX = abs(entity[0] - self.x)
        entityDY = abs(entity[1] - self.y)
        if entityDX <= range and entityDY <= range:
            return True
        return False

    def render(self, win, scroll, dt, time, show_outline):
        if time % 2:
            if self.frame + 1 >= self.frames*3:
                self.frame = 0
            else:
                self.frame += 1
        
        if show_outline:
            pygame.draw.rect(win, (255, 0, 0), self.rect, 1, border_radius=1)

        coords = self.rect.topleft - scroll
        if self.moving:
            if self.last_direction == "right":
                win.blit(self.anim_walk_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_walk_left[self.frame//3], coords)
        else:
            if self.last_direction == "right":
                win.blit(self.anim_idle_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_idle_left[self.frame//3], coords)
        
        if self.on_render != None:
            self.on_render(win, scroll, dt, time, show_outline)



class ExtraFunctions:
    def __init__(self, obj):
        self.obj = obj

    def merchant_init(self):
        self.shop_open = False
        self.items = []
        self.generate_items()
        self.inv_box_size = (50,50)
        self.next_slot = 1
        self.selected_slot = 0
        self.dragging_item = None
        self.hover_rect = None
        self.rendered_items = []
        self.buy_btn = None

    def generate_items(self):
        x = 1
        for item in self.obj.items.items:
            if self.obj.items.items[item]["sold-by-merchant"]:
                item_obj = Item(item, self.obj.items.items[item], self.obj.items.art[item])
                self.items.append({"item": item_obj, 
                "amount": 99, "price": self.obj.items.items[item]["price"], "slot": x, "img": item_obj.image})
                x += 1

    def merchant_update(self, dt, player, scroll):
        if self.shop_open == False:
            if pygame.mouse.get_pressed()[0] and mouse_over(pygame.mouse.get_pos(), self.obj.rect.left-scroll.x, self.obj.rect.top-scroll.y, self.obj.rect.width+4, self.obj.rect.height+4):
                self.shop_open = True
                return True
        return None

    def merchant_on_key(self, key):
        if key == pygame.K_ESCAPE:
            self.shop_open = False
            return False
        return None

    def merchant_render(self, win, scroll, dt, time, show_outline):
        if self.shop_open:
            self.render_inv(win)

    def render_inv(self, win):
        win_size = win.get_size()
        y = 0
        x = 1
        init_x = (win_size[0]/2-(self.inv_box_size[0]*10/2))-300
        init_y = win_size[1]/3
        self.hover_rect = None
        for z in range(30):
            color = INV_NOT_EMPTY if self.check_slot(z+1) else INV_EMPTY
            rect = pygame.draw.rect(win, color,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]))

            outline_rect = pygame.draw.rect(win, INV_SELECTED if self.selected_slot == z+1 else BLACK,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]), 4)

            self.draw_item(win, z+1, rect)
            if mouse_over(pygame.mouse.get_pos(), outline_rect.left, outline_rect.top, outline_rect.width, outline_rect.height):
                self.hover_rect = (rect, z+1) # This is needed so that it draws the hover box on top of the inventory
                if self.dragging_item != None: # dragging item rect outline
                    outline_rect = pygame.draw.rect(win, INV_HOVER,
                    (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
                    self.inv_box_size[0], self.inv_box_size[1]), 4)

            if str(x).endswith("0") and x != 0:
                y += self.inv_box_size[1]+4.1
                x = 1
            else:
                x += 1
        
        self.render_buy_btn(win)
        self.preview_box(win)
        if self.hover_rect != None:
            self.hover_item(self.hover_rect[0], self.hover_rect[1], win)
        
        if self.dragging_item != None:
            img = self.dragging_item["img"].copy()
            img.fill((255, 255, 255, 200), None, pygame.BLEND_RGBA_MULT)
            win.blit(img, (pygame.mouse.get_pos()[0]-img.get_width()/2, pygame.mouse.get_pos()[1]-img.get_height()/2))

    def render_buy_btn(self, win):
        win_size = win.get_size()
        x = (win_size[0]/2-(self.inv_box_size[0]*10/2))-245
        y = win_size[1]/3-59

        font = pygame.font.Font('assets/font.ttf', 32-len("Buy"))
        info_box = pygame.Rect(x, y, 52, 32)
        txt_surface = font.render("Buy", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED, info_box, 7) 
        self.buy_btn = rect

    def preview_box(self, win):
        win_size = win.get_size()
        item = self.check_slot(self.selected_slot)
        rect = pygame.draw.rect(win, BLACK,
        (int(win_size[0]-500), int(win_size[1]/3), 
        350, 400))

        if item != False:
            w = 200
            h = 200
            win.blit(pygame.transform.scale(item["img"], (w, h)), (rect.center[0]-w/2+15, rect.center[1]-h/1.5))    
            draw_text(win, (rect.center[0]-w/2+15, rect.center[1]-h/1.5-24), item["item"].name, 24, WHITE, "assets/font.ttf")
            #draw_text(win, (rect.left+20, rect.center[1]+40), item["item"].item["description"], 20, WHITE, "assets/font.ttf")
            font = pygame.font.Font("assets/font.ttf", 20)
            draw_large_text(win, item["item"].item["description"], (rect.left+20, rect.center[1]+40), font, WHITE)
            draw_text(win, (rect.left+20, rect.center[1]+80), "Type: " + item["item"].item["type"], 20, WHITE, "assets/font.ttf")

            item_dict = item["item"].item
            x, y = rect.left+20, rect.center[1]+100
            for effect in item_dict["effects"]:
                draw_text(win, (x, y), f"{effect.capitalize()}: " + str(item_dict[effect]), 20, WHITE, "assets/font.ttf")
                y += 20 
            draw_text(win, (x, y), f"Price: {item_dict['price']}", 20, WHITE, "assets/font.ttf")

    def on_click(self, btn, inventory, player):
        try:
            if btn == 1: # Left click
                if self.shop_open:
                    if self.buy_btn != None and self.check_slot(self.selected_slot) != False:
                        if mouse_over(pygame.mouse.get_pos(), self.buy_btn.left, self.buy_btn.top, self.buy_btn.width, self.buy_btn.height):
                            item = self.check_slot(self.selected_slot)["item"]
                            if player.balance - item.item["price"] < 0:
                                alert(text=f'You only have {player.balance} and the item costs {item.item["price"]}', title='Not enough money', button='OK')
                                return None
                            confirm_ = confirm(text=f'Do you want to buy {item.name} for {item.item["price"]}', title='Buy Item', buttons=['Yes', 'No'])
                            if confirm_ == 'Yes':
                                player.balance -= item.item["price"]
                                inventory.add_item(self.check_slot(self.selected_slot)["item"])
                                if self.check_slot(self.selected_slot)["amount"] == 1:
                                    self.items.remove(self.check_slot(self.selected_slot))
                                else:
                                    self.check_slot(self.selected_slot)["amount"] -= 1

                    # Selecting a slot
                    if self.hover_rect != None:
                        if mouse_over(pygame.mouse.get_pos(), self.hover_rect[0].left, self.hover_rect[0].top, self.hover_rect[0].width+4, self.hover_rect[0].height+4):
                            self.selected_slot = self.hover_rect[1]
            return None
        except TypeError:
            return None

    def check_slot(self, slot):
        for i in self.items:
            if i["slot"] == slot:
                return i
        return False

    def get_item(self, item_id):
        for i in self.items:
            if i["item"].id == item_id:
                return i
        return False

    def draw_item(self, win, slot, rect):
        item = self.check_slot(slot)
        if item:
            win.blit(item["img"], (rect.center[0]-item["item"].width/2, rect.center[1]-item["item"].height/2))
            if len(str(item["amount"])) == 1:
                offset = (16, 24)
            elif len(str(item["amount"])) == 2:
                offset = (20, 24)
            elif len(str(item["amount"])) == 3:
                offset = (28, 24)
            draw_text(win, (rect.right-offset[0], rect.bottom-offset[1]), str(item["amount"]), 16, BLACK, "assets/opensans.ttf")
        else:
            pass

    def hover_item(self, rect, slot, win):
        item = self.check_slot(slot)
        if mouse_over(pygame.mouse.get_pos(), rect.left, rect.top, rect.width, rect.height):
            if item and self.dragging_item == None:
                text = item["item"].name
                color = (255, 255, 255)
                font = pygame.font.Font('assets/font.ttf', 32-len(text))
                info_box = pygame.Rect(rect.left+30, rect.top-30, 140, 32)
                txt_surface = font.render(text, True, color)
                win.blit(txt_surface, (info_box.x+5, info_box.y+5))
                pygame.draw.rect(win, color, info_box, 3) 