import pygame
from python.particles import Particle


class Player():
    def __init__(self, screen_dimensions, player_coords, particles):
        self.particles = particles
        self.x = player_coords[0]
        self.y = player_coords[1]
        self.width = 48
        self.height = 48
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        self.speed = 4
        self.health = 50
        self.damage = 15
        self.max_health = 100
        self.balance = 100
        self.s_width = screen_dimensions[0]
        self.s_height = screen_dimensions[1]
        self.pistol = pygame.transform.scale(pygame.image.load("assets/items/pistol.png").convert_alpha(), (32,32))
        self.pistol_reversed = pygame.transform.scale(pygame.image.load("assets/items/pistol reversed.png").convert_alpha(), (32,32))
        self.ammo = 100
        self.loaded_ammo = 10
        self.gun_equiped = False
        self.since_gun_equiped = 0
        self.bullets = []
        self.load_animations()
        self.dead = False
        # variables for the tutorial
        self.moved = False
        self.opened_inv = False
        self.used_item = False
        self.opened_chest = False
        self.shot = False
        self.attacked = False

    def load_animations(self):
        self.anim_idle_right = []
        self.anim_idle_left = []
        self.anim_walk_right = []
        self.anim_walk_left = []
        self.frame = 0
        self.moving = False
        self.last_direction = "right"
        self.last_world_direction = None

        for x in range(1, 5):
            idle = pygame.transform.scale(pygame.image.load(f"assets/player/idle{x}.png").convert_alpha(), (self.width, self.height))
            walk = pygame.transform.scale(pygame.image.load(f"assets/player/run{x}.png").convert_alpha(), (self.width, self.height))

            self.anim_idle_right.append(idle)
            self.anim_idle_left.append(pygame.transform.flip(idle, True, False))
            self.anim_walk_right.append(walk)
            self.anim_walk_left.append(pygame.transform.flip(walk, True, False))
        
    def die(self):
        self.dead = True

    def update(self, dt, enemies):
        self.move()
        for bullet in self.bullets:
            x = bullet.update(dt, enemies, self)
            if x:
                self.bullets.remove(bullet)

    def attack(self): # make gold particles when enemy killed to get coins
        if self.loaded_ammo <= 0:
            if self.ammo > 0:
                self.ammo -= 10
                self.loaded_ammo += 10
        else:
            self.loaded_ammo -= 1
            self.bullets.append(Bullet(self.x+self.width/2+15, self.y+self.height-12, self.last_direction))

    def move(self):
        keys = pygame.key.get_pressed()
        if keys[pygame.K_d] or keys[pygame.K_RIGHT]:
            if self.x > 100*32-32:
                return
            self.last_direction = "right"
            self.moving = True
            self.rect.x += self.speed
            self.x += self.speed
            self.last_y_direction = None
            if keys[pygame.K_w] or keys[pygame.K_UP]:
                if self.y < 0:
                    return
                self.rect.y -= self.speed
                self.y -= self.speed
                self.last_world_direction = "up"
            elif keys[pygame.K_s] or keys[pygame.K_DOWN]:
                if self.y > 100*32-32:
                    return
                self.rect.y += self.speed
                self.y += self.speed
                self.last_y_direction = "down"
            self.moved = True
        elif keys[pygame.K_a] or keys[pygame.K_LEFT]:
            if self.x < 0:
                return
            self.last_direction = "left"
            self.moving = True
            self.rect.x -= self.speed
            self.x -= self.speed
            self.last_y_direction = None
            if keys[pygame.K_w] or keys[pygame.K_UP]:
                if self.y < 0:
                    return
                self.rect.y -= self.speed
                self.y -= self.speed
                self.last_y_direction = "up"
            elif keys[pygame.K_s] or keys[pygame.K_DOWN]:
                if self.y > 100*32-32:
                    return
                self.rect.y += self.speed
                self.y += self.speed
                self.last_y_direction = "down"
            self.moved = True
        elif keys[pygame.K_w] or keys[pygame.K_UP]:
            if self.y < 0:
                return
            self.moving = True
            self.rect.y -= self.speed
            self.y -= self.speed
            self.last_y_direction = "up"
            self.moved = True
        elif keys[pygame.K_s] or keys[pygame.K_DOWN]:
            if self.y > 100*32-32:
                return
            self.moving = True
            self.rect.y += self.speed
            self.y += self.speed
            self.last_y_direction = "down"
            self.moved = True
        else:
            self.last_y_direction = None
            self.moving = False


    def render(self, win, scroll, dt, time, show_outline):
        if time % 2:
            if self.frame + 1 >= 12:
                self.frame = 0
            else:
                self.frame += 1
        
        if show_outline:
            pygame.draw.rect(win, (255, 0, 0), self.rect, 1, border_radius=1)

        coords = self.rect.topleft - scroll
        if self.moving:
            if self.last_direction == "right":
                win.blit(self.anim_walk_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_walk_left[self.frame//3], coords)
        else:
            if self.last_direction == "right":
                win.blit(self.anim_idle_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_idle_left[self.frame//3], coords)

        if self.gun_equiped:
            if self.last_direction == "right":
                win.blit(self.pistol, (coords[0]+self.width/2, coords[1]+self.height/2))
            elif self.last_direction == "left":
                win.blit(self.pistol_reversed, (coords[0]-self.width/2+12, coords[1]+self.height/2)) 
            self.since_gun_equiped += 1
        if round(self.since_gun_equiped*dt) >= 160:
            self.gun_equiped = False
            self.since_gun_equiped = 0
        
        for bullet in self.bullets:
            bullet.render(win, scroll)


class Bullet():
    def __init__(self, x, y, direction):
        self.x = x
        self.y = y 
        self.direction = direction
        self.speed = 12
        self.color = (255, 255, 255)
        self.time = 0

    def render(self, win, scroll):
        pygame.draw.circle(win, self.color, (self.x - scroll.x, self.y - scroll.y), 2)

    def update(self, dt, enemies, player):
        for enemy in enemies.enemies_spawned:
            if enemy.rect.collidepoint(self.x, self.y):
                enemy.health -= player.damage
                if enemy.health <= 0:
                    enemy.die(enemies, player)
                    player.attacked = True
                return True
        if self.direction == "right":
            self.x += self.speed
        elif self.direction == "left":
            self.x -= self.speed
        self.time += 1
        if round(self.time * dt) >= 120:
            return True
        return False